#include "graphics.h"
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>


Vector2i::Vector2i(int xx, int yy):
    x(xx),
    y(yy)
{
}

Screen::Screen(int width, int height, string gamename):
    m_height(height),
    m_width(width),
    m_gamename(gamename)
{
    m_screen = init(width,height,gamename);
}

Screen::~Screen()
{
    close();
}

Texture::Texture(string ffilename):
    filename(ffilename)
{
    texture = IMG_Load(ffilename.c_str());
}

Texture::~Texture()
{
    SDL_FreeSurface(texture);
}

TileSprite::TileSprite(Texture& ttexture, Vector2i ttilePos):
    texture(&ttexture),
    tilePos(ttilePos)
{
}

void TileSprite::Draw(Screen& screen)
{
    SDL_Rect temp;
    temp.x = tilePos.x * texture->texture->w;
    temp.y = tilePos.y * texture->texture->h;

    SDL_BlitSurface(texture->texture,0,screen.GetSurface(),&temp);
}

SDL_Surface* Screen::init(int width, int height, string gamename)
{
    SDL_Init(SDL_INIT_VIDEO);
    IMG_Init(IMG_INIT_PNG);
    SDL_Surface* screen = SDL_SetVideoMode(width, height, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
    SDL_WM_SetCaption(gamename.c_str(),NULL);

    return screen;
}

void Screen::close()
{
    SDL_Quit();
    IMG_Quit();
}
