#include <cstdio>
#include <string>
#include <cassert>
using namespace std;

#define SCREEN_WIDTH 768
#define SCREEN_HEIGHT 622
#define FIRST_LEVEL 0

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <vector>
#include <sstream>

#include "graphics.h"
#include "escapetilemap.h"
#include "player.h"

int main(int argc, char* argv[])
{
    TTF_Init();
    // LOAD RESOURCES
    Screen screen(SCREEN_WIDTH,SCREEN_HEIGHT, "Es Cape!");

    vector<string> maplist( {
                            "maps/1.txt",
                            "maps/2.txt",
                            "maps/3.txt",
                            "maps/4.txt",
                            "maps/5.txt",
                            "maps/6.txt",
                            "maps/7.txt",
                            "maps/8.txt",
                            "maps/9.txt",
                            "maps/10.txt",
                            "maps/11.txt",
                            "maps/12.txt",
                            "maps/13.txt",
                            "maps/14.txt",
                            "maps/15.txt"
                             } );
    int currentMap = FIRST_LEVEL;

    Texture wall_graphic    ("assets/wall.bmp");
    Texture ridge_graphic   ("assets/wallridge.bmp");
    Texture floor_graphic   ("assets/floor.bmp");
    Texture pit_graphic     ("assets/pit.bmp");
    Texture pitedge_graphic ("assets/pitedge.bmp");
    Texture start_graphic   ("assets/start.bmp");
    Texture end_graphic     ("assets/end.bmp");
    Texture fence_graphic   ("assets/fence.bmp");
    Texture spike_graphic   ("assets/spikes.bmp");
    Texture statue_graphic  ("assets/statue.bmp");
    Texture flow_graphic    ("assets/wallridgeflow.bmp");
    Texture water_graphic   ("assets/water.bmp");
    Texture ladder_graphic  ("assets/ladder.bmp");
    Texture light_graphic   ("assets/light.bmp");
    Texture guardleft_graphic    ("assets/guardleft.bmp");
    Texture guardright_graphic   ("assets/guardright.bmp");
    Texture goldwall_graphic     ("assets/goldwall.bmp");
    Texture goldstatue_graphic   ("assets/goldstatue.bmp");
    Texture pillar_graphic       ("assets/pillar.bmp");
    Texture carpet_graphic       ("assets/carpet.bmp");
    Texture cape_graphic         ("assets/cape.bmp");
    Texture clamp_graphic        ("assets/wallclamps.bmp");
    Texture bed_graphic          ("assets/bed.bmp");


    // CREATE GAME ENTITIES
    EscapeTileMap myMap;
    myMap.addDefinition('0',wall_graphic);
    myMap.addDefinition('.',floor_graphic);
    myMap.addDefinition('M',ridge_graphic);
    myMap.addDefinition('X',pit_graphic);
    myMap.addDefinition('"',pitedge_graphic);
    myMap.addDefinition('@',start_graphic);
    myMap.addDefinition('!',end_graphic);
    myMap.addDefinition('T',fence_graphic);
    myMap.addDefinition('^',spike_graphic);
    myMap.addDefinition('P',statue_graphic);
    myMap.addDefinition('Y',flow_graphic);
    myMap.addDefinition('~',water_graphic);
    myMap.addDefinition('H',ladder_graphic);
    myMap.addDefinition('o',light_graphic);
    myMap.addDefinition('d',guardleft_graphic);
    myMap.addDefinition('b',guardright_graphic);
    myMap.addDefinition('W',goldwall_graphic);
    myMap.addDefinition('G',goldstatue_graphic);
    myMap.addDefinition('I',pillar_graphic);
    myMap.addDefinition('#',carpet_graphic);
    myMap.addDefinition('C',cape_graphic);
    myMap.addDefinition('U',clamp_graphic);
    myMap.addDefinition('m',bed_graphic);

    myMap.load(maplist[currentMap]);

    EscapePlayer player(2,2);
    myMap.spawnPoint = player.sprite.tilePos;

    bool hasCompletedGame = false;
    bool isAtFive = false;
    bool isPlaying = true;
    bool atTitleScreen = true;

    SDL_Surface* titlescreen = IMG_Load("assets/title.bmp");
    SDL_Rect topLeft; topLeft.x = topLeft.y = 0;
    SDL_BlitSurface(titlescreen,NULL,screen.GetSurface(),&topLeft);
    screen.flip();

    while(atTitleScreen)
    {
        SDL_Event event;
        while(SDL_PollEvent(&event))
        {
            switch(event.type)
            {
            case SDL_QUIT:
                atTitleScreen = false;
                isPlaying = false;
                break;
            case SDL_KEYDOWN:
                atTitleScreen = false;
                break;
            default:
                break;
            }
        }
    }

    Uint32 startTime = SDL_GetTicks();

    // MAIN LOOP
    while(isPlaying)
    {
        // HANDLE THE IMPUTS
        SDL_Event event;
        while(SDL_PollEvent(&event))
        {
            switch(event.type)
            {
                case SDL_QUIT:
                isPlaying = false;
                break;
                case SDL_KEYDOWN:
                switch(event.key.keysym.sym)
                {
                    case SDLK_UP:
                    if (!myMap.isWall(player.sprite.tilePos.x,player.sprite.tilePos.y -1))
                        player.move(direction::up);
                    break;
                    case SDLK_DOWN:
                    if (!myMap.isWall(player.sprite.tilePos.x,player.sprite.tilePos.y +1))
                        player.move(direction::down);
                    break;
                    case SDLK_RIGHT:
                    if (!myMap.isWall(player.sprite.tilePos.x +1,player.sprite.tilePos.y))
                        player.move(direction::right);
                    break;
                    case SDLK_LEFT:
                    if (!myMap.isWall(player.sprite.tilePos.x -1,player.sprite.tilePos.y))
                        player.move(direction::left);
                    break;
                    case SDLK_SPACE:
                    player.sprite.tilePos = myMap.spawnPoint;
                    break;
                    default:
                    break;
                }
                break;
                default:
                break;

            }
        }
        if (myMap.isBad(player.sprite.tilePos.x,player.sprite.tilePos.y))
        {
            player.setState(EscapePlayer::hurt);
        }

        if (myMap.isWater(player.sprite.tilePos.x,player.sprite.tilePos.y))
        {
            player.setState(EscapePlayer::wet);
        }
        // DRAW
        myMap.draw(screen);
        player.sprite.Draw(screen);

        //DON'T FLIP ME BRO
        screen.flip();

        if (myMap.isBad(player.sprite.tilePos.x,player.sprite.tilePos.y))
        {
            SDL_Delay(250);
            SDL_Event e;
            while (SDL_PollEvent(&e)){}
            player.sprite.tilePos = myMap.spawnPoint;
            player.setState(EscapePlayer::walk1);
        }
        if (myMap.isStairs(player.sprite.tilePos.x,player.sprite.tilePos.y))
        {
            currentMap ++;
            SDL_Delay(500);
            SDL_Event e;
            while (SDL_PollEvent(&e)){}
            if((unsigned int)currentMap < maplist.size())
            {
                myMap.load(maplist[currentMap]);
                myMap.spawnPoint = player.sprite.tilePos;
            }
            else
            {
                hasCompletedGame = true;
                isPlaying = false;
                break;
            }
        }
    }

    // TODO: Add game complete background
    if(hasCompletedGame)
    {
        Uint32 timePassedms = SDL_GetTicks() - startTime;
        float timePassedSeconds = timePassedms/1000.0;
        int timePassedMinutes = timePassedSeconds/60;

        stringstream ss;
        string mminutes = (timePassedMinutes == 1 ? "minute" : "minutes");
        string sseconds = (timePassedSeconds == 1 ? "second" : "seconds");
        ss << "Time taken: " << timePassedMinutes << " " << mminutes <<" and " << timePassedSeconds - timePassedMinutes*60 <<  " " << sseconds;

        TTF_Font* textfont = TTF_OpenFont("fonts/arial.ttf",24);
        SDL_Color textcolor = { 255, 255, 255 };
        SDL_Surface* textsurface = TTF_RenderText_Solid(textfont,ss.str().c_str(),textcolor);

        Texture winScreen_graphic("assets/win.bmp");
        TileSprite winScreen(winScreen_graphic);

        SDL_Rect textpos;
        textpos.x = screen.GetWidth()/2 - textsurface->w/2;
        textpos.y = screen.GetHeight() - textsurface->h;

        winScreen.Draw(screen);
        SDL_BlitSurface(textsurface,NULL,screen.GetSurface(),&textpos);
        screen.flip();

        SDL_Event event;

        bool isWaiting = true;
        while(isWaiting)
        {
            SDL_WaitEvent(&event);
            switch(event.type)
            {
            case SDL_QUIT:
                isWaiting = false;
                break;
            default:
                break;
            }
        }

        SDL_FreeSurface(textsurface);
        TTF_CloseFont(textfont);
    }

    TTF_Quit();
    SDL_FreeSurface(titlescreen);
    return 0;
}
